--------------------------------------------------------------------------------
-- Title:               LUList.lua
-- Description:         Like a square peg in a round hole
-- Author:              Raphaël Szwarc http://alt.textdrive.com/lua/
-- Creation Date:       February 1, 2005
-- Legal:               Copyright (C) 2005 Raphaël Szwarc
--------------------------------------------------------------------------------

-- import dependencies
require( "LUObject" )

-- parent class
local superClass = LUObject

-- inherit the super class methods
local thisClass = superClass.inherit( superClass )

-- factory class method, this is where all the instance methods are implemented as closures
thisClass.new = function()
        -- instantiate the super class
        local super = superClass.new()
        
        -- inherit the super class instance methods
        local this = super.class().inherit( super )
        
        -- local table for the instance variables
        local ivars = {}
        
        -- instance method to add an object to this list
        this.add = function( anObject )
                if anObject ~= nil then
                        table.insert( ivars, anObject )
                end
                
                return this
        end
        
        -- instance method to add all the values of the given list to this list
        this.addAll = function( aList )
                if this.isKindOf( aList ) == true then
                        local count = aList.size()
                        
                        for index = 1, count do
                                local   aValue = aList.get( index )
                                
                                this.add( aValue )
                        end
                elseif type( aList ) == 'table' then
                        local count = table.getn( aList )
 
                        for index = 1, count do
                                local   aValue = aList[ index ]
                                
                                this.add( aValue )
                        end
                end
               
                return this
        end
        
        -- instance method to clear this list
        this.clear = function()
                local count = this.size()
                
                for index = 1, count do
                        table[ index ] = nil
                end

                ivars = {}
                
                return this
        end

        -- instance method to compare the given object with this list for equality
        this.equals = function( anObject )
                if this == anObject then
                        return true
                end
                
                if this.isKindOf( anObject ) == true then
                        local count = this.size()
                
                        if count == anObject.size() then
                                for index = 1, count do
                                        local aValue = this.get( index )
                                        local anotherValue = anObject.get( index )

                                        if this.isKindOf( aValue ) == true and aValue.equals( anotherValue ) == false then
                                                return false
                                        end
                                        
                                        if aValue ~= anotherValue then
                                                return false
                                        end
                                end
                                
                                return true
                        end
                end
                
                return false
        end

        -- instance method to get an object at a given index
        this.get = function( anIndex )
                if type( anIndex ) ~= "number" then
                        anIndex =  this.indexOf( anIndex )
                end
                
                if anIndex > 0 and anIndex <= this.size() then
                        return ivars[ anIndex ]
                end
                
                return nil
        end

        -- instance method to get the index of an object
        this.indexOf = function( anObject )
                if anObject then
                        local isKindOf = this.isKindOf( anObject )
                        local count = this.size()
                        
                        for index = 1, count do
                                local anotherObject = this.get( index )
                                
                                if anObject == anotherObject then
                                        return index
                                end
                                
                                if isKindOf == true and anObject.equals( anotherObject ) == true then
                                        return index
                                end
                        end
                end
                
                return -1
        end

        -- instance method to remove from this list
        this.remove = function( anIndex )
                if type( anIndex ) ~= "number" then
                        anIndex =  this.indexOf( anIndex )
                end
                
                local anObject = ivars[ anIndex ]
        
                table.remove( ivars, anIndex )
                
                return anObject
        end

        -- instance method to access this list size
        this.size = function()
                return table.getn( ivars )
        end

        -- instance method for string representation
        this.toString = function()
                local aTable = {}
                local count = this.size()
                
                table.insert( aTable, "( " )
                
                if count > 0 then
                        for index = 1, count do
                                local anObject = this.get( index )
                                local aDescription = nil
                                
                                if this.isKindOf( anObject ) == true then
                                        aDescription = anObject.toString()
                                else
                                        aDescription = tostring( anObject )
                                end

                                table.insert( aTable, string.format( "%q", aDescription ) )
                                
                                table.insert( aTable, ", " )
                        end

                        table.remove( aTable, table.getn( aTable ) )
                end
                
                table.insert( aTable, " )" )
                
                return table.concat( aTable, "" )
        end

        -- return the public instance methods
        return this
end

-- define the class name
LUList = thisClass

return LUList
