--------------------------------------------------------------------------------
-- Title:               LUObject.lua
-- Description:         Like a square peg in a round hole
-- Author:              Raphaël Szwarc http://alt.textdrive.com/lua/
-- Creation Date:       February 1, 2005
-- Legal:               Copyright (C) 2005 Raphaël Szwarc
--------------------------------------------------------------------------------

-- parent class
local superClass = nil

-- inherit the super class methods
local thisClass = {}

-- factory class method, this is where all the instance methods are implemented as closures
thisClass.new = function()
        -- instantiate the super class
       local super = nil

        -- inherit the super class instance methods
        local this = {}
        
        -- instance method to access the class methods from an instance
        this.class = function() 
                return thisClass
        end

        -- instance method for equality
        this.equals = function( anObject )
                if this == anObject then
                        return true
                end
        
                return false
        end

        -- instance method for hashing
        this.hashCode = function()
                return this.class().hashCode( this );
        end

        -- instance method to check if the given object is indeed an object
        this.isKindOf = function( anObject )
                return this.class().isKindOf( anObject )
        end

        -- instance method to access itself
        this.self = function()
                return this
        end

        -- instance method to access super
        this.super = function()
                return super
        end

        -- instance method for string representation
        this.toString = function()
                return this.class().toString()..":"..this.hashCode()
        end

        -- return the public instance methods
        return this;
end

-- class method to set the inheritance chain
thisClass.inherit = function( super )
        local   someMethods = {};

        setmetatable( someMethods, super )
        super.__index = super
        
        return someMethods;
end

-- class method to access the parent class
thisClass.class = function()
        return superClass
end

-- class method for equality
thisClass.equals = function( anObject )
        if thisClass == anObject then
                return true
        end

        return false
end

-- class method for hashing
thisClass.hashCode = function( anObject )
        if anObject == nil then
                anObject = thisClass
        end

        return tonumber( string.gsub( tostring( anObject ), type( anObject )..": ", "" ), 16 )
end

-- class method to check if the given object is indeed an object
thisClass.isKindOf = function( anObject )
        if type( anObject ) == "table" and type( anObject.equals ) == "function" and anObject.equals( anObject ) == true then
                return true
        end

        return false
end

-- class method to access itself
thisClass.self = function()
        return thisClass;
end

-- class method to access super
thisClass.super = function()
        return superClass
end


-- class method for string representation
thisClass.toString = function()
        return "@"..thisClass.hashCode()
end

-- define the class name
LUObject = thisClass

return LUObject
